function [L2EMG_diag] = CalculateL2EMG_diag(param, FileList,canSkip)

% CalculateL2EMG_diag calculates diagonal dense L2EMG descriptors
% with PCA-SIFT. 
%
% Peihua Li, Qilong Wang, Hui Zeng, and Lei Zhang. Local Log-Euclidean 
% Multivariate Gaussian Descriptor and Its Application to Image Classification. 
% IEEE Transactions on Pattern Analysis and Machine Intelligence, 2016.
% DOI: 10.1109/TPAMI.2016.2560816
% 
% Please cite the paper above if you use the code:
%
% For questions,  please conact:  Peihua  Li  (Email:  peihuali at dlut dot edu dot cn ), 
%                                               Qilong Wang (Email: qlwang at mail dot dlut dot edu dot cn) 
%
% The software is provided ''as is'' and without warranty of any kind,
% experess, implied or otherwise, including without limitation, any
% warranty of merchantability or fitness for a particular purpose.

fprintf('****************************************************************\n');
fprintf('Calculate Diagonal L2EMG \n');
fprintf('****************************************************************\n');
start_time = round(clock);
L2EMG_diag = [];
if canSkip == 0
    parfor f =1:length(FileList.L2EMG_diag_FileList)
        if ~exist(FileList.L2EMG_diag_FileList{f})
            features = CalculateFeature(FileList.imageFileList{f},param);    
            [~] = CalculateL2EMG_diag_perImage(features, param.Diag_L2EMG.patchsize, param.Diag_L2EMG.step, param.pca, FileList.L2EMG_diag_FileList{f});
        end
    end
else
    fprintf('Skipped\n');
end

fprintf('Completed.\n');
end_time = round(clock);
elapsed_time = end_time - start_time;
fprintf('Time taken: %5d days %5d hours %5d minutes %5d seconds\n\n', elapsed_time(3), elapsed_time(4), elapsed_time(5), elapsed_time(6));


function [L2EMG] = CalculateL2EMG_diag_perImage(features, patch_size, step, pca, currentFilePath)

EmbeddingFlag = 1;

hgt = features.imageHeight;
wid = features.imageWidth;
scales = length(patch_size);
for p = 1:scales
    xmin = 1;
    ymin = 1;
    xmax = xmin + patch_size(p) - 1;
    ymax = ymin + patch_size(p) - 1;
    num_threshold = patch_size(p)*patch_size(p)*0.125;
    nx = floor((wid-xmin)/step+1);
    ny = floor((hgt-ymin)/step+1);
    sel_x = cell(nx,1);
    sel_y = cell(ny,1);
    L2EMG.imageHeight = hgt;
    L2EMG.imageWidth = wid;
    L2EMG.x{p} = zeros(nx*ny,1);
    L2EMG.y{p} = zeros(nx*ny,1);
    L2EMG.patchsize{p} = ones(nx*ny,1) * patch_size(p);
    L2EMG.mask{p} = zeros(nx*ny,1);
    L2EMG.des{p} = zeros(nx*ny,2*pca.dim_sift,'single');
    count_x = 0;
    count_y = 0;
    while(ymin<hgt)
        count_y = count_y + 1;
        sel_y{count_y} = find(ymin<=features.y & features.y<=ymax);
        ymin = ymin + step;
        ymax = ymax + step;
    end
    while(xmin<wid)
        count_x = count_x + 1;
        sel_x{count_x} = find(xmin<=features.x & features.x<=xmax);
        xmin = xmin + step;
        xmax = xmax + step;
    end

    ymin = 1;
    ymax = ymin + patch_size(p) - 1;
    count_y = 0;
    count_xy = 0;
    count_zero = 0;
    while(ymin<hgt)
        count_x = 0;
        count_y = count_y + 1;
        xmin = 1;
        xmax = xmin + patch_size(p) - 1;
        while(xmin<wid)
            count_xy = count_xy + 1;
            count_x = count_x + 1;
            sel = intersect(sel_x{count_x},sel_y{count_y});
            des = features.fea(sel,:);
            patch_mask = features.mask(sel);
            des = des(patch_mask,:);
            if size(des,1) > num_threshold
                covariances = std(des)+1e-3;
                means = mean(des);
                if EmbeddingFlag == 1
                    embeded_covariances = log(covariances);
                    idx = find(covariances == 1);
                    embeded_means = embeded_covariances ./ (covariances - 1) .* means;
                    embeded_means(idx) = means(idx);
                    y = [embeded_means,embeded_covariances];
                else
                    y = [means,covariances];
                end
                L2EMG.des{p}(count_xy,:)  =  y;
                L2EMG.mask{p}(count_xy) = 1;
            else
                count_zero = count_zero + 1;
                L2EMG.des{p}(count_xy,:) = zeros(1,2*pca.dim_sift); 
                L2EMG.mask{p}(count_xy) = 0;
            end
            L2EMG.x{p}(count_xy) = (xmin+xmax-1)/2;
            L2EMG.y{p}(count_xy) = (ymin+ymax-1)/2;
            xmin = xmin + step;
            xmax = xmax + step;
        end
        ymin = ymin + step;
        ymax = ymax + step;
    end
end
L2EMG.x = cat(1,L2EMG.x{:});
L2EMG.y = cat(1,L2EMG.y{:});
L2EMG.patchsize = cat(1,L2EMG.patchsize{:});
L2EMG.mask = cat(1,L2EMG.mask{:});
L2EMG.des = cat(1,L2EMG.des{:});
sp_make_dir(currentFilePath);
save(currentFilePath,'L2EMG');






