function [sift_logP,sift_cov_mask, grid_x, grid_y] = dense_cov_CM(I_orientation, patch_size, grid_spacing)
%Generate the dense grid of L2ECM descriptors for each image

% Input:
% I_orientation: raw features
% gridSpacing: the spacing for the grid to be used when generating the L2ECM  descriptors
% patchSize: the patch size used for generating the L2ECM descriptors

% Peihua Li and Qilong Wang, Local Log-Euclidean covariance matrix
% (L2ECM) for image representation and its applications, in Proc.
% Eur. Conf. Comput. Vis. (ECCV), 2012, pp. 469C482.

% Please cite the papers above if you use the code:
%
% For questions,  please conact:  Peihua  Li  (Email:  peihuali at dlut dot edu dot cn ), 
%                                               Qilong Wang (Email: qlwang at mail dot dlut dot edu dot cn) 
%
% The software is provided ''as is'' and without warranty of any kind,
% experess, implied or otherwise, including without limitation, any
% warranty of merchantability or fitness for a particular purpose.

[hgt wid num_angles] = size(I_orientation);
num_bins = patch_size;

grid_x = patch_size/2:grid_spacing:wid-patch_size/2;
grid_y = patch_size/2:grid_spacing:hgt-patch_size/2;

% Sample SIFT bins at valid locations (without boundary artifacts)
% find coordinates of sample points (bin centers)
[sample_x, sample_y] = meshgrid(linspace(1,patch_size+1,num_bins+1));
sample_x = sample_x(1:num_bins,1:num_bins); sample_x = sample_x(:)-patch_size/2;
sample_y = sample_y(1:num_bins,1:num_bins); sample_y = sample_y(:)-patch_size/2;


sift_arr = zeros([length(grid_y) length(grid_x) num_angles*num_bins*num_bins]);
sift_logP = zeros([length(grid_y) length(grid_x) (num_angles)*(num_angles)]);

b = 0;
for n = 1:num_bins*num_bins
    sift_arr(:,:,b+1:b+num_angles) = I_orientation(grid_y+sample_y(n), grid_x+sample_x(n), :);
    b = b+num_angles;
end


one_pixel_data = zeros(num_angles*num_bins*num_bins,1);
one_pixel_mat  = zeros(num_angles, patch_size*patch_size);
data_cov       = zeros(num_angles, num_angles);
sift_cov_mask  = zeros([length(grid_y) length(grid_x)],'uint8');

d = num_angles;

diag_S = zeros(d, 1);
U = zeros(d, d);
S = U;
V = U;
logP = U;
for y=1:size(sift_arr,1)
    for x=1:size(sift_arr,2)
        one_pixel_data = squeeze(sift_arr(y,x,:));
        one_pixel_mat  = reshape(one_pixel_data,num_angles, patch_size*patch_size);

        one_pixel_mat = one_pixel_mat'; 
        data_cov       = cov(one_pixel_mat);
        
        [U S V] = svd(data_cov);
        
        diag_S = diag(S);
        valid_flag = (length(find(diag_S>eps(4))) == d);
        sift_cov_mask(y, x) = valid_flag;
        if valid_flag 
            logP = U*diag(log(diag_S))*V';
        else
            logP = U*diag(log(max(diag_S, eps(4))))*V';
        end
        sift_logP(y,x,:)   = logP(:);
   end
end

