function [param] = set_parameter(image_dataset)

% Set all the parameters for the proposed methods
% main parameters:
% SIFT: raw feature for diag L2EMG
%       raw_feature: raw feature type for diag L2EMG, i.e. 'sift'(default), 'esift', 'ehog', 'L2ECM', 'L2EMG'
%       multiScale_type: two type of multiScale choice, 'ImageScale' only for sift which resizes the image at different scales
%                        'PatchSize' for other feature tpyes which fix the image size and change the patch size of raw feature 
%       step: sampling step of raw feature
% PCA: pca is performed for raw feature, diag_L2EMG and full_L2EMG,respectively
% Diag_L2EMG:
%            patchsize: patch size of Diag_L2EMG, we use single scale by default
%            step: sampling step of Diag_L2EMG
% Full_L2EMG:
%            patchsize: patch size of Full_L2EMG, we use single scale by default
%            step: sampling step of Full_L2EMG
%            PCAstep: sampling step of Full_L2EMG for training PCA, larger step for efficiency
%            EmbeddingType: there are three tpyes of embedding in our paper
%            i.e. 'IE-LogE-L'Eq.(16)  'IE-LogE-R' Eq.(18)  'DE-LogE' Eq.(21)                          
% GMM: number of Gaussians for Fisher Vector coding
% SPM: spatial pyramid match setting

% Peihua Li, Qilong Wang, Hui Zeng, and Lei Zhang. Local Log-Euclidean 
% Multivariate Gaussian Descriptor and Its Application to Image Classification. 
% IEEE Transactions on Pattern Analysis and Machine Intelligence, 2016.
% DOI: 10.1109/TPAMI.2016.2560816
%
% Peihua Li and Qilong Wang, Local Log-Euclidean covariance matrix
% (L2ECM) for image representation and its applications, in Proc.
% Eur. Conf. Comput. Vis. (ECCV), 2012, pp. 469C482.

% Please cite the papers above if you use the code:
%
% For questions,  please conact:  Peihua  Li  (Email:  peihuali at dlut dot edu dot cn ), 
%                                               Qilong Wang (Email: qlwang at mail dot dlut dot edu dot cn) 
%
% The software is provided ''as is'' and without warranty of any kind,
% experess, implied or otherwise, including without limitation, any
% warranty of merchantability or fitness for a particular purpose.


 switch image_dataset
    case {'caltech256','scene15'}
        param.maxImageSize = 300; 
    case {'voc07','sun397'}
        param.maxImageSize = 480; 
     case {'fmd'}
        param.maxImageSize = 600;      
    otherwise, error('Unknown dataset type');
 end
 %% SIFT
 param.sift.raw_feature =  'sift'; % sift; esift; ehog; elbp; L2ECM; L2EMG 
 param.sift.mutliScale_type = 'ImageScale';    % PatchSize   ImageScale
 if strcmp(param.sift.mutliScale_type,'ImageScale') == 1 % currently only for sift
     param.sift.step = 4;
     param.sift.scales = [2.^(1:-0.5:-1)];
 else
     param.sift.patch_size = [2 4 6 8]; 
     param.sift.step = 2;
 end
 param.sift.contrast = 0.005;
 %% PCA
 param.pca.total_feature_num = 1e6; 
 param.pca.dim_sift = 64;
 param.pca.dim_L2EMG_diag = 64;
 param.pca.dim_L2EMG_full = 96;
 param.pca.PCAflag = 1;    % true means pca, false means no pca
 param.pca.whiteningflag = 0;
 %% Diag L2EMG
 param.Diag_L2EMG.patchsize=[16];   % Single Scale L2EMG (Mutli-scale using e.g., [12,16,20,24])
 param.Diag_L2EMG.step=[4];
 %% Full L2EMG
 param.Full_L2EMG.patchsize=[16];   % Single Scale L2EMG (Mutli-scale using e.g., [12,16,20,24])
 param.Full_L2EMG.step=[2];
 param.Full_L2EMG.PCAstep=[4];
 param.Full_L2EMG.embeddingType='IE-LogE-L';   % IE-LogE-L  IE-LogE-R   DE-LogE
 %% GMM
 param.numClusters = 256;
 %% SPM
 switch image_dataset
    case {'fmd'}
        param.block_x =[1];
        param.block_y =[1];
    case {'voc07','scene15','sun397','caltech256'}
        param.block_x =[1;1];
        param.block_y =[1;3];
    otherwise, error('Unknown dataset type');
 end
 param.block_xy =  param.block_x .* param.block_y; 
 param.num_pyramid = sum(param.block_xy);
 param.pyramidLevels = length(param.block_xy);

end
 
